(function () {
    'use strict';

    var painter, meter, world, b2d;

    /**
     * Setup references to Box2D classes for easier use in future
     */
    function setRefs() {
        b2d = {
            b2Vec2: Box2D.Common.Math.b2Vec2,
            b2BodyDef: Box2D.Dynamics.b2BodyDef,
            b2Body: Box2D.Dynamics.b2Body,
            b2FixtureDef: Box2D.Dynamics.b2FixtureDef,
            b2World: Box2D.Dynamics.b2World,
            b2PolygonShape: Box2D.Collision.Shapes.b2PolygonShape,
            b2CircleShape: Box2D.Collision.Shapes.b2CircleShape
        };
    }

    /**
     * Initialization
     */
    function init() {
        meter = new FPSMeter({
            graph: 1,
            history: 20
        });

        setRefs();

        world = new b2d.b2World(new b2d.b2Vec2(0, 10), true);

        painter = new Painter(document.getElementById('main'));

        addWalls();
        addBalls();

        loop();
    }

    /**
     * Add 100 balls to stage
     */
    function addBalls() {
        var i, j;

        for (i = 0; i < 20; i++) {
            for (j = 1; j <= 12; j++) {
                createBall(i * 6 + (j % 2 === 0 ? 5 : 8), j * 5);
            }
        }
    }

    /**
     * Add walls to stage
     */
    function addWalls() {
        /*
        x1, y1, w1, h1, x2, y2 ... wn, hn
         */
        var walls = [
                0, 0, 128, 1,
                0, 1, 1, 70,
                127, 1, 1, 70,
                0, 71, 128, 1
            ],
            i = 0;

        while (walls[i] !== undefined) {
            createWall(walls[i], walls[i + 1], walls[i + 2], walls[i + 3]);
            i += 4;
        }
    }

    /**
     * Create a wall with given position and size
     * @param  {Number} x X coordinate
     * @param  {Number} y Y coordinate
     * @param  {Number} w Width
     * @param  {Number} h Height
     */
    function createWall(x, y, w, h) {
        var bdef, body, fdef;

        bdef = new b2d.b2BodyDef();
        bdef.position = new b2d.b2Vec2(x + w / 2, y + h / 2);
        bdef.type = b2d.b2Body.b2_staticBody;
        body = world.CreateBody(bdef);
            
        fdef = new b2d.b2FixtureDef();
        fdef.shape = new b2d.b2PolygonShape();
        fdef.shape.SetAsBox(w / 2, h / 2);
        fdef.restitution = 1;
        fdef.friction = 0;
        body.CreateFixture(fdef);
    }

    /**
     * Create a ball at given position
     * @param  {Number} x X coordinate
     * @param  {Number} y Y coordinate
     */
    function createBall(x, y) {
        var bdef, body, fdef;

        bdef = new b2d.b2BodyDef();
        bdef.position = new b2d.b2Vec2(x, y);
        bdef.type = b2d.b2Body.b2_dynamicBody;
        body = world.CreateBody(bdef);

        fdef = new b2d.b2FixtureDef();
        fdef.shape = new b2d.b2CircleShape();
        fdef.shape.SetRadius(2);
        fdef.restitution = 1;
        fdef.friction = 0;
        body.CreateFixture(fdef);
    }

    /**
     * Main loop
     */
    function loop() {
        meter.tickStart();
        world.Step(1 / 60, 8, 8);
        world.ClearForces();
        painter.paint(world);
        requestAnimationFrame(loop);
        meter.tick();
    }

    window.addEventListener('load', init);
})();


// Adapted from https://gist.github.com/paulirish/1579671 which derived from 
// http://paulirish.com/2011/requestanimationframe-for-smart-animating/
// http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating

// requestAnimationFrame polyfill by Erik Möller.
// Fixes from Paul Irish, Tino Zijdel, Andrew Mao, Klemen Slavič, Darius Bacon

// MIT license

if (!Date.now) Date.now = function() { return new Date().getTime(); };

(function() {
    var vendors = ['webkit', 'moz'];
    for (var i = 0; i < vendors.length && !window.requestAnimationFrame; ++i) {
        var vp = vendors[i];
        window.requestAnimationFrame = window[vp+'RequestAnimationFrame'];
        window.cancelAnimationFrame = (window[vp+'CancelAnimationFrame'] ||
            window[vp+'CancelRequestAnimationFrame']);
    }
    if (/iP(ad|hone|od).*OS 6/.test(window.navigator.userAgent) ||
        !window.requestAnimationFrame || !window.cancelAnimationFrame) {
        var lastTime = 0;
        window.requestAnimationFrame = function(callback) {
            var now = Date.now();
            var nextTime = Math.max(lastTime + 16, now);
            return setTimeout(function() { callback(lastTime = nextTime); },
                              nextTime - now);
        };
        window.cancelAnimationFrame = clearTimeout;
    }
}());