(function () {
    'use strict';

    var meter, world, phys2D, painter;

    /**
     * Initialization
     */
    function init() {
        meter = new FPSMeter({
            graph: 1,
            history: 20
        });

        phys2D = new Physics2DDevice();

        painter = new Painter(document.getElementById('main'));

        world = phys2D.createWorld({
            gravity : [0, 10],
            velocityIterations : 8,
            positionIterations : 8,
        });

        addWalls();
        addBalls();

        loop();
    }

    /**
     * Add 100 balls to stage
     */
    function addBalls() {
        var i, j;

        for (i = 0; i < 20; i++) {
            for (j = 1; j <= 12; j++) {
                createBall(i * 6 + (j % 2 === 0 ? 5 : 8), j * 5);
            }
        }
    }

    /**
     * Add walls to stage
     */
    function addWalls() {
        /*
        x1, y1, w1, h1, x2, y2 ... wn, hn
         */
        var walls = [
                0, 0, 128, 1,
                0, 1, 1, 70,
                127, 1, 1, 70,
                0, 71, 128, 1
            ],
            i = 0;

        while (walls[i] !== undefined) {
            createWall(walls[i], walls[i + 1], walls[i + 2], walls[i + 3]);
            i += 4;
        }
    }

    /**
     * Create a wall with given position and size
     * @param  {Number} x X coordinate
     * @param  {Number} y Y coordinate
     * @param  {Number} w Width
     * @param  {Number} h Height
     */
    function createWall(x, y, w, h) {
        var body = phys2D.createRigidBody({
            type: 'static',
            shapes: [
                phys2D.createPolygonShape({
                    vertices: phys2D.createRectangleVertices(0, 0, w, h),
                    material: phys2D.createMaterial({
                        elasticity: 1,
                        staticFriction: 0,
                        dynamicFriction: 0,
                        rollingFriction: 0
                    })
                })
            ],
            position: [x, y]
        });

        world.addRigidBody(body);
    }

    /**
     * Create a ball at given position
     * @param  {Number} x X coordinate
     * @param  {Number} y Y coordinate
     */
    function createBall(x, y) {
        var body;

        body = phys2D.createRigidBody({
            type: 'dynamic',
            shapes: [
                phys2D.createCircleShape({
                    radius: 2,
                    material: phys2D.createMaterial({
                        elasticity: 1,
                        staticFriction: 0,
                        dynamicFriction: 0,
                        rollingFriction: 0
                    })
                })
            ],
            linearDrag: 0,
            angularDrag: 0,
            position: [x, y]
        });

        world.addRigidBody(body);
    }

    /**
     * Main loop
     */
    function loop() {
        meter.tickStart();
        world.step(1 / 60);
        painter.paint(world.rigidBodies);
        requestAnimationFrame(loop);
        meter.tick();
    }

    window.addEventListener('load', init);
})();


// Adapted from https://gist.github.com/paulirish/1579671 which derived from 
// http://paulirish.com/2011/requestanimationframe-for-smart-animating/
// http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating

// requestAnimationFrame polyfill by Erik Möller.
// Fixes from Paul Irish, Tino Zijdel, Andrew Mao, Klemen Slavič, Darius Bacon

// MIT license

if (!Date.now) Date.now = function() { return new Date().getTime(); };

(function() {
    var vendors = ['webkit', 'moz'];
    for (var i = 0; i < vendors.length && !window.requestAnimationFrame; ++i) {
        var vp = vendors[i];
        window.requestAnimationFrame = window[vp+'RequestAnimationFrame'];
        window.cancelAnimationFrame = (window[vp+'CancelAnimationFrame'] ||
            window[vp+'CancelRequestAnimationFrame']);
    }
    if (/iP(ad|hone|od).*OS 6/.test(window.navigator.userAgent) ||
        !window.requestAnimationFrame || !window.cancelAnimationFrame) {
        var lastTime = 0;
        window.requestAnimationFrame = function(callback) {
            var now = Date.now();
            var nextTime = Math.max(lastTime + 16, now);
            return setTimeout(function() { callback(lastTime = nextTime); },
                              nextTime - now);
        };
        window.cancelAnimationFrame = clearTimeout;
    }
}());